const pmx             = require('pmx');
const pm2             = require('pm2');
const debug           = require('debug')('pm2:service-discovery');
const ServicesManager = require('./ServicesManager');
const http             = require('http');
const url             = require('url');

class App {

  constructor(config) {
    this.config          = config;
    this.servicesManager = new ServicesManager(config);
    this.server          = null;
    this.attachPM2();
  }

  reload() {
    this.servicesManager.emit('reload');
  }

  attachPM2() {
    pm2.connect(() => {
      pm2.launchBus((err, bus) => {
        bus.on('process:event', (data) => {
          if (data.event === 'online') {
            updateService(data.process.pm_id, this.servicesManager.services);
          }
        });
      });

      this.servicesManager.on('loaded', services => updateServices(services));
      this.servicesManager.loadServices();
    });
  }

  start() {
    this.server = http.createServer((req, res) => {
      const path = url.parse(req.url).pathname;

      switch(path) {
        case '/ready':
          if(this.servicesManager.status.status === 'ready') {
            res.writeHead(204);
          } else {
            res.writeHead(500);
          }
          break;
        case '/services':
          res.writeHead(200, { "Content-Type": "application/json" });
          res.write(JSON.stringify(this.servicesManager.services));
          break;
        case '/status':
          res.writeHead(200, { "Content-Type": "application/json" });
          res.write(JSON.stringify(this.servicesManager.getStatus()));
          break;
        default:
          res.writeHead(404, { "Content-Type": "application/json" });
          res.write(JSON.stringify({ message: `No route for "${path}"`}))
      }
      res.end();
    });

    const port = this.config.port || 20000;

    this.server.listen(port, (err) => {
      if (err) return debug('something bad happened', err);
      debug(`server is listening on ${port}`)
    });
  }
}

function updateServices(services) {
  pm2.list((err, rs) => {
    rs.forEach(proc => {
      if (proc.pm2_env.status === 'online') {
        updateService(proc.pm2_env.pm_id, services);
      }
    })
  });
}

function updateService(pmId, services) {
  pm2.sendDataToProcessId(pmId, {
      type: 'process:msg',
      data: services || [],
      topic: 'service:update',
    },
    function (err, res) {
      if (err) console.log(err);
    });
}

module.exports = App;