const Eventemitter = require('eventemitter2').EventEmitter2;
const rp           = require('request-promise');
const request      = require('request');
const debug        = require('debug')('pm2:service-discovery');

class ServicesManager extends Eventemitter {

  constructor(config) {
    super();
    this.config = config;
    this.status = {
      startedAt: new Date(),
      updatedAt: null,
      status: 'init',
      lastError: null,
    };

    this.services = {};

    this.on('load', () => this.loadServices());
    this.on('reload', () => this.loadServices());
    this.on('error', debug);

    if (this.config.watch === true) this.watchServices();
  }

  loadServices() {
    return rp
      .get(`${this.config.uri}/v1/catalog/services`)
      .then(res => JSON.parse(res))
      .then(services => {
        const promises = [];

        for ( let index in services ) {
          const p = rp
            .get(`${this.config.uri}/v1/catalog/service/${index}`)
            .then(res => JSON.parse(res))
            .then(services => {
              debug (`New service ${index}`);
              this.services[index] = services;
            });
          promises.push(p)
        }

        return Promise.all(promises)
      })
      .then(() => this.status.updatedAt = new Date())
      .then(() => this.status.status = 'ready')
      .then(() => this.emit('loaded', this.services))
      .catch(err => {
        this.status.status    = 'errored';
        this.status.lastError = err;
        this.emit('error', err);
      })
  }

  watchServices() {
    const index = this.version ? `index=${this.version}` : '';
    const uri   = `${this.config.uri}/v1/catalog/services?${index}`;

    if (this.rq) this.rq.abort();

    this.rq = request({ uri }, (err, res) => {
      if (err) return this.emit('error', err);

      this.version = res.headers['x-consul-index'];
      this.loadServices();
      this.watchServices();
    });
  }

  getStatus() {
    return this.status;
  }
}

module.exports = ServicesManager;